// IIRDlg.cpp : implementation file
//

#include "stdafx.h"
#include "dsp.h"
#include "IIRDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "filters.h"
#include "PoleZeroDlg.h"
#include "PoleZeroWnd.h"

/////////////////////////////////////////////////////////////////////////////
// CIIRDlg dialog


CIIRDlg::CIIRDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CIIRDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CIIRDlg)
	m_GraphType = 1;
	m_Normalize = TRUE;
	m_ApplyTo = -1;
	//}}AFX_DATA_INIT
	cur_pole = 0;
	cur_zero = 0;
}

void CIIRDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIIRDlg)
	DDX_Control(pDX, IDC_DELZERO, m_DelZero);
	DDX_Control(pDX, IDC_DELPOLE, m_DelPole);
	DDX_Control(pDX, IDC_ADDZERO, m_AddZero);
	DDX_Control(pDX, IDC_ADDPOLE, m_AddPole);
	DDX_Control(pDX, IDC_ZEROS, m_ZerosList);
	DDX_Control(pDX, IDC_POLES, m_PolesList);
	DDX_Radio(pDX, IDC_GRAPHTYPE, m_GraphType);
	DDX_Check(pDX, IDC_NORMALIZE, m_Normalize);
	DDX_CBIndex(pDX, IDC_APPLYTO, m_ApplyTo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIIRDlg, CDialog)
	//{{AFX_MSG_MAP(CIIRDlg)
	ON_BN_CLICKED(IDC_ADDPOLE, OnAddpole)
	ON_BN_CLICKED(IDC_DELPOLE, OnDelpole)
	ON_BN_CLICKED(IDC_ADDZERO, OnAddzero)
	ON_BN_CLICKED(IDC_DELZERO, OnDelzero)
	ON_BN_CLICKED(IDC_SAVE, OnSave)
	ON_BN_CLICKED(IDC_LOAD, OnLoad)
	ON_LBN_DBLCLK(IDC_POLES, OnDblclkPoles)
	ON_LBN_DBLCLK(IDC_ZEROS, OnDblclkZeros)
	ON_BN_CLICKED(IDC_DISPLAYGRAPH, OnDisplaygraph)
	ON_BN_CLICKED(IDC_CLEARLISTS, OnClearlists)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIIRDlg message handlers

void CIIRDlg::AddPole(double r, double w)
{
	char s[32];
	rp[cur_pole] = r;
	wp[cur_pole] = w;
	cur_pole++;
	sprintf(s, "r = %.03g, w = %.03g", r, w);
	m_PolesList.AddString(s);
	m_AddPole.EnableWindow(cur_pole < MAX_POLES);
	m_DelPole.EnableWindow();
}

void CIIRDlg::AddZero(double r, double w)
{
	char s[32];
	r0[cur_zero] = r;
	w0[cur_zero] = w;
	cur_zero++;
	sprintf(s, "r = %.03g, w = %.03g", r, w);
	m_ZerosList.AddString(s);
	m_AddZero.EnableWindow(cur_zero < MAX_ZEROS);
	m_DelZero.EnableWindow();
}

void CIIRDlg::DelPole(int index)
{
	m_PolesList.DeleteString(index);
	for (int i = index; i < cur_pole - 2; i++)
	{
		rp[i] = rp[i + 1];
		wp[i] = wp[i + 1];
	}
	cur_pole--;
	m_DelPole.EnableWindow(cur_pole > 0);
	m_AddPole.EnableWindow();
}

void CIIRDlg::DelZero(int index)
{
	m_ZerosList.DeleteString(index);
	for (int i = index; i < cur_zero - 2; i++)
	{
		r0[i] = r0[i + 1];
		w0[i] = w0[i + 1];
	}
	cur_zero--;
	m_DelZero.EnableWindow(cur_zero > 0);
	m_AddZero.EnableWindow();
}


BOOL CIIRDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	
	UpdateButtons();
	LoadLists();

	if (!m_ApplyTo)
	{
		CComboBox *cb = (CComboBox *)GetDlgItem(IDC_APPLYTO);
		cb->DeleteString(1);
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CIIRDlg::OnAddpole() 
{
	// TODO: Add your control notification handler code here
	CPoleZeroDlg d;
	d.m_HalfRate = m_HalfRate;
	if (d.DoModal() != IDOK) return;
	AddPole(d.m_R, d.m_W);
}

void CIIRDlg::OnDelpole() 
{
	// TODO: Add your control notification handler code here
	int s = m_PolesList.GetCurSel();
	if (s >= 0) DelPole(s);
}

void CIIRDlg::OnAddzero() 
{
	// TODO: Add your control notification handler code here
	CPoleZeroDlg d;
	d.m_HalfRate = m_HalfRate;
	if (d.DoModal() != IDOK) return;
	AddZero(d.m_R, d.m_W);
}

void CIIRDlg::OnDelzero() 
{
	// TODO: Add your control notification handler code here
	int s = m_ZerosList.GetCurSel();
	if (s >= 0) DelZero(s);	
}

void CIIRDlg::OnSave() 
{
	// TODO: Add your control notification handler code here
	CFileDialog s(false, "fil", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"    (*.iir)|*.iir|  (*.*)|*.*||",
		this);

	if (s.DoModal() != IDOK) return;
	if (!UpdateData()) return;

	CFile f(s.GetPathName(), CFile::modeCreate | CFile::modeWrite);
	CArchive ar(&f, CArchive::store);

	ar.Write(&m_Normalize, sizeof(m_Normalize));
	ar.Write(&m_GraphType, sizeof(m_GraphType));
	ar.Write(&cur_pole, sizeof(cur_pole));
	ar.Write(&cur_zero, sizeof(cur_zero));
	ar.Write(rp, cur_pole * sizeof(double));
	ar.Write(wp, cur_pole * sizeof(double));
	ar.Write(r0, cur_zero * sizeof(double));
	ar.Write(w0, cur_zero * sizeof(double));
	
	ar.Close();
	f.Close();
}

void CIIRDlg::OnLoad() 
{
	// TODO: Add your control notification handler code here
	CFileDialog o(true, "fil", NULL, OFN_HIDEREADONLY,
		"    (*.iir)|*.iir|  (*.*)|*.*||",
		this);

	if (o.DoModal() != IDOK) return;
	
	CFile f(o.GetPathName(), CFile::modeRead);
	CArchive ar(&f, CArchive::load);

	ar.Read(&m_Normalize, sizeof(m_Normalize));
	if (m_Normalize != 0 && m_Normalize != 1)
		AfxThrowArchiveException(CArchiveException::badIndex, o.GetPathName());
	ar.Read(&m_GraphType, sizeof(m_GraphType));
	if (m_GraphType < 0 || m_GraphType > 3)
		AfxThrowArchiveException(CArchiveException::badIndex, o.GetPathName());
	ar.Read(&cur_pole, sizeof(cur_pole));
	if (cur_pole >= MAX_POLES)
		AfxThrowArchiveException(CArchiveException::badIndex, o.GetPathName());
	ar.Read(&cur_zero, sizeof(cur_zero));
	if (cur_zero >= MAX_ZEROS)
		AfxThrowArchiveException(CArchiveException::badIndex, o.GetPathName());
	ar.Read(rp, cur_pole * sizeof(double));
	ar.Read(wp, cur_pole * sizeof(double));
	ar.Read(r0, cur_zero * sizeof(double));
	ar.Read(w0, cur_zero * sizeof(double));
	
	ar.Close();
	f.Close();

	LoadLists();
	UpdateButtons();
	UpdateData(false);
}

void CIIRDlg::LoadLists()
{
	int n = m_PolesList.GetCount();
	for (int i = 0; i < n; i++)
		m_PolesList.DeleteString(0);

	n = m_ZerosList.GetCount();
	for (i = 0; i < n; i++)
		m_ZerosList.DeleteString(0);

	for (i = 0; i < cur_pole; i++)
	{
		char s[32];
		sprintf(s, "r = %.03g, w = %.03g", rp[i], wp[i]);
		m_PolesList.AddString(s);
	}

	for (i = 0; i < cur_zero; i++)
	{
		char s[32];
		sprintf(s, "r = %.03g, w = %.03g", r0[i], w0[i]);
		m_ZerosList.AddString(s);
	}
}

void CIIRDlg::UpdateButtons()
{
	m_AddPole.EnableWindow(cur_pole < MAX_POLES);
	m_AddZero.EnableWindow(cur_zero < MAX_ZEROS);
	m_DelPole.EnableWindow(cur_pole != 0);
	m_DelZero.EnableWindow(cur_zero != 0);
}

void CIIRDlg::OnDblclkPoles() 
{
	// TODO: Add your control notification handler code here
	CPoleZeroDlg d;
	int sel = m_PolesList.GetCurSel();
	char s[32];
	d.m_HalfRate = m_HalfRate;
	d.m_R = rp[sel];
	d.m_W = wp[sel];
	if (d.DoModal() != IDOK) return;
	rp[sel] = d.m_R;
	wp[sel] = d.m_W;
	sprintf(s, "r = %.03g, w = %.03g", rp[sel], wp[sel]);
	m_PolesList.DeleteString(sel);
	m_PolesList.InsertString(sel, s);
}

void CIIRDlg::OnDblclkZeros() 
{
	// TODO: Add your control notification handler code here
	CPoleZeroDlg d;
	int sel = m_ZerosList.GetCurSel();
	char s[32];
	d.m_HalfRate = m_HalfRate;
	d.m_R = r0[sel];
	d.m_W = w0[sel];
	if (d.DoModal() != IDOK) return;
	r0[sel] = d.m_R;
	w0[sel] = d.m_W;
	sprintf(s, "r = %.03g, w = %.03g", r0[sel], w0[sel]);
	m_ZerosList.DeleteString(sel);
	m_ZerosList.InsertString(sel, s);	
}

void CIIRDlg::OnDisplaygraph() 
{
	// TODO: Add your control notification handler code here
	double *a = new double[MAX_ZEROS + 3];
	double *b = new double[MAX_POLES + 3];
	double *d = new double[R_LENGTH];
	int num_a = (cur_zero << 1) + 1;
	int num_b = cur_pole << 1, i;
	char graph_name[64];
	
	UpdateData();
	GetPoleZeroCoeff(rp, wp, r0, w0, a, b, cur_pole, cur_zero, m_Normalize == 1);
	GetImpulseResponse(num_a, a, num_b, b, R_LENGTH, d);

	if (m_GraphType == 1 || m_GraphType == 2)
		for (i = 0; i < R_LENGTH; i++)
		{
			Complex z;
			z.FromPolar(1, M_PI * i / R_LENGTH);
			Complex c = GetTransferFuncValue(z, rp, wp, r0, w0, cur_pole, cur_zero);
			if (m_GraphType == 1)
				d[i] = c.Mod();
			else
				d[i] = c.Arg();
		}

	switch (m_GraphType)
	{
		case 0: strcpy(graph_name, " "); break;
		case 1: strcpy(graph_name, " "); break;
		case 2: strcpy(graph_name, " "); break;
		case 3: strcpy(graph_name, "  Z-");
	}
	if (m_GraphType < 3)
		strcat(graph_name, " ");

	CPoleZeroWnd *wnd = new CPoleZeroWnd();
	wnd->m_HalfRate = m_HalfRate;
	wnd->m_GraphType = m_GraphType;
	wnd->m_GraphWidth = R_LENGTH;
	wnd->m_GraphHeight = R_LENGTH >> 1;
	wnd->SetData(d, R_LENGTH);
	if (m_GraphType == 3)
		wnd->SetPoleZeroCoords(rp, wp, r0, w0, cur_pole, cur_zero);
	wnd->Create(NULL, graph_name, CS_HREDRAW | CS_VREDRAW, 
		CRect(0, 0, R_LENGTH + 140, (R_LENGTH >> 1) + 140), NULL, 0);
	wnd->ShowWindow(SW_SHOW);
	wnd->UpdateWindow();

	delete d;
	delete b;
	delete a;
}

void CIIRDlg::OnClearlists() 
{
	// TODO: Add your control notification handler code here
	cur_pole = 0;
	cur_zero = 0;
	LoadLists();
	UpdateButtons();
}

void CIIRDlg::OnOK() 
{
	// TODO: Add extra validation here
	for (int i = 0; i < cur_pole; i++)
		if (rp[i] >= 1.0)
		{
			if (MessageBox(" ,     \n \
   \n\n ?", 
				"", MB_YESNO) == IDNO)
				return;
			else
				break;
		}

	CDialog::OnOK();
}
