// PoleZeroWnd.cpp : implementation file
//

#include "stdafx.h"
#include "dsp.h"
#include "PoleZeroWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define AXES_SPACE	20

/////////////////////////////////////////////////////////////////////////////
// CPoleZeroWnd

IMPLEMENT_DYNCREATE(CPoleZeroWnd, CFrameWnd)

CPoleZeroWnd::CPoleZeroWnd()
{
	d = NULL;
}

CPoleZeroWnd::~CPoleZeroWnd()
{
}


BEGIN_MESSAGE_MAP(CPoleZeroWnd, CFrameWnd)
	//{{AFX_MSG_MAP(CPoleZeroWnd)
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPoleZeroWnd message handlers

BOOL CPoleZeroWnd::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class
	cs.style = WS_DLGFRAME | WS_CAPTION | WS_SYSMENU;
	return CFrameWnd::PreCreateWindow(cs);
}

int CPoleZeroWnd::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, (long)GetStockObject(WHITE_BRUSH));
	SetClassLong(m_hWnd, GCL_HICON, (long)LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDR_MAINFRAME)));
	CenterWindow();
	
	return 0;
}

void CPoleZeroWnd::SetData(double *data, int length)
{
	d = new double[length];
	memcpy(d, data, sizeof(double) * length);
}

void CPoleZeroWnd::SetPoleZeroCoords(double *rp, double *wp, double *r0, double *w0, int poles, int zeros)
{
	int i;
	max_r = 0.0;
	num_poles = poles;
	num_zeros = zeros;
	memcpy(CPoleZeroWnd::rp, rp, sizeof(double) * poles);
	memcpy(CPoleZeroWnd::wp, wp, sizeof(double) * poles);
	memcpy(CPoleZeroWnd::r0, r0, sizeof(double) * zeros);
	memcpy(CPoleZeroWnd::w0, w0, sizeof(double) * zeros);
	for (i = 0; i < poles; i++)
		if (rp[i] > max_r) max_r = rp[i];
	for (i = 0; i < zeros; i++)
		if (r0[i] > max_r) max_r = r0[i];
}

void CPoleZeroWnd::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CRect r;
	GetClientRect(&r);
	int y = r.Height() >> 1, y_, i;
	CPen *old_pen, *pen;
	int x = (r.Width() - m_GraphWidth) >> 1;
	double max = -1.7E+308;
	bool Centered = true;
	r.left = x;
	r.right = x + m_GraphWidth + AXES_SPACE - 1;
	r.top = y - (m_GraphHeight >> 1) - AXES_SPACE;
	r.bottom = y + (m_GraphHeight >> 1) + AXES_SPACE - 1;
	
	// TODO: Add your message handler code here
	for (i = 0; i < m_GraphWidth; i++)
		if (d[i] > max) max = d[i];
	switch (m_GraphType)
	{
		case 0: 
			DrawAxes(&dc, r, m_GraphWidth, max, 5, "x", "A[x]", Centered);
			break;
		case 1:
			Centered = false;
			r.bottom -= AXES_SPACE;
			DrawAxes(&dc, r, m_HalfRate, max, 5, "f", "Amp[f]", Centered);
			break;
		case 2:
			max = M_PI;
			DrawAxes(&dc, r, m_HalfRate, M_PI, 5, "f", "Phase[f]", Centered);
			break;
		case 3: 
			DrawZPlane(&dc);
			return;
	}
	
	pen = new CPen();
	pen->CreatePen(PS_SOLID, 2, RGB(0, 0, 128));
	old_pen = dc.SelectObject(pen);

	x += 2;
	if (Centered)
		y = r.top + (m_GraphHeight >> 1);
	else
		y = r.top + m_GraphHeight;
	y += AXES_SPACE;
	for (i = 0; i < m_GraphWidth; i++)
	{
		y_ = int(d[i] / max * m_GraphHeight);
		if (Centered) y_ >>= 1;
		if (!i)
			dc.MoveTo(i + x, y - y_);
		else
			dc.LineTo(i + x, y - y_);
	}

	dc.SelectObject(old_pen);
	delete pen;

	// Do not call CFrameWnd::OnPaint() for painting messages
}

void CPoleZeroWnd::DrawAxes(CDC *dc, CRect r, double max_x, double max_y, int tick_freq, char *x_label, char *y_label, bool Centered)
{
	int x, y, y2, i;
	double inc = double(r.Width() - AXES_SPACE) / tick_freq;
	double inc2;
	char s[20];

	a_maxx = max_x;
	a_rect = r;
	strcpy(a_xlabel, x_label);
	strcpy(a_ylabel, y_label);

	dc->MoveTo(r.left, r.bottom);
	dc->LineTo(r.left, r.top);
	dc->MoveTo(r.left - 5, r.top + 5);
	dc->LineTo(r.left, r.top);
	dc->LineTo(r.left + 6, r.top + 6);
	if (Centered)
	{
		y = r.top + (r.Height() >> 1);
		inc2 = double((r.Height() >> 1) - AXES_SPACE) / tick_freq;
	}
	else
	{
		y = r.bottom;
		inc2 = double(r.Height() - AXES_SPACE) / tick_freq;
	}
	dc->MoveTo(r.left, y);
	dc->LineTo(r.right, y);
	dc->MoveTo(r.right - 5, y - 5);
	dc->LineTo(r.right, y);
	dc->LineTo(r.right - 6, y + 6);

	for (i = 1; i <= tick_freq; i++)
	{
		sprintf(s, "%d", int(i * max_x / tick_freq));
		x = r.left + int(i * inc);
		dc->MoveTo(x, y - 2);
		dc->LineTo(x, y + 3);
		dc->SetTextAlign(TA_CENTER);
		dc->TextOut(x, y + 5, s);
		dc->SetTextAlign(TA_RIGHT);
		sprintf(s, "%.03g", i * max_y / tick_freq);
		x = r.left;
		if (Centered)
			y2 = r.top + (r.Height() >> 1) - int(i * inc2);
		else
			y2 = r.top + r.Height() - int(i * inc2);
		dc->MoveTo(x - 2, y2);
		dc->LineTo(x + 3, y2);
		dc->TextOut(x - 5, y2 - 7, s);
		if (Centered)
		{
			sprintf(s, "-%.03g", i * max_y / tick_freq);
			y2 = r.top + (r.Height() >> 1) + int(i * inc2);
			dc->MoveTo(x - 2, y2);
			dc->LineTo(x + 3, y2);
			dc->TextOut(x - 5, y2 - 7, s);
		}
	}
	if (Centered)
		y = r.top + (r.Height() >> 1);
	else
		y = r.top + r.Height();
	dc->TextOut(x - 5, y - 7, "0");
	dc->SetTextAlign(TA_LEFT);
	dc->TextOut(r.left + 10, r.top - 20, y_label);
	dc->TextOut(r.left + r.Width() + 10, y - 7, x_label);
}

void DrawPoleZero(CDC *dc, CRect rect, double r, double w, double max_r, bool IsPole, bool DrawText)
{
	int r2 = int(r / max_r * ((rect.Width() >> 1) - AXES_SPACE));
	CRect rect2;
	CPen *pen = new CPen;
	int c = 0x80, x, y;
	if (!IsPole) c += 0x40;
	pen->CreatePen(PS_SOLID, 1, RGB(c, c, c));
	char s[32];

	dc->SelectObject(pen);
	if (!DrawText)
	{
		rect2.top = rect.bottom - r2;
		rect2.bottom = rect.bottom + r2;
		rect2.left = (rect.Width() >> 1) - r2;
		rect2.right = (rect.Width() >> 1) + r2;
		dc->Arc(rect2, CPoint(rect2.right, rect.bottom), CPoint(rect2.left, rect.bottom));
	}
	else
	{
		x = (rect.Width() >> 1) + int(r2 * cos(w));
		y = rect.bottom - int(r2 * sin(w));
		dc->MoveTo(rect.Width() >> 1, rect.bottom);
		dc->LineTo(x, y);
		dc->SelectStockObject(BLACK_PEN);
		if (IsPole)
		{
			dc->MoveTo(x - 3, y - 3);
			dc->LineTo(x + 4, y + 4);
			dc->MoveTo(x - 3, y + 3);
			dc->LineTo(x + 4, y - 4);
		}
		else
			dc->Ellipse(x - 3, y - 3, x + 3, y + 3);
		sprintf(s, "r = %.03g, w = %.03g", r, w);
		if (IsPole)
			dc->SetTextColor(RGB(0xFF, 0, 0));
		else
			dc->SetTextColor(RGB(0, 0, 0xFF));
		dc->TextOut(x + 5, y - 15, s);
	}
	dc->SelectStockObject(BLACK_PEN);
	delete pen;
}

void CPoleZeroWnd::DrawZPlane(CDC *dc)
{
	CRect rect;
	GetClientRect(&rect);
	int i;

	dc->SetBkMode(TRANSPARENT);
	dc->SetTextAlign(TA_LEFT);
	for (i = 0; i < num_poles; i++)
		DrawPoleZero(dc, rect, rp[i], wp[i], max_r, true, false);
	for (i = 0; i < num_zeros; i++)
		DrawPoleZero(dc, rect, r0[i], w0[i], max_r, false, false);
	for (i = 0; i < num_poles; i++)
		DrawPoleZero(dc, rect, rp[i], wp[i], max_r, true, true);
	for (i = 0; i < num_zeros; i++)
		DrawPoleZero(dc, rect, r0[i], w0[i], max_r, false, true);
	dc->SetTextColor(0);
	dc->TextOut(rect.right - AXES_SPACE + 5, rect.bottom - 15, "0");
	dc->SetTextAlign(TA_RIGHT);
	dc->TextOut(AXES_SPACE - 5, rect.bottom - 15, "Pi");
	dc->SetTextAlign(TA_CENTER);
	dc->TextOut(rect.Width() >> 1, rect.bottom - (rect.Width() >> 1), "Pi/2");
	dc->MoveTo(AXES_SPACE, rect.bottom - 1);
	dc->LineTo(rect.right - AXES_SPACE, rect.bottom - 1);
	dc->MoveTo(rect.Width() >> 1, rect.bottom);
	dc->LineTo(rect.Width() >> 1, rect.bottom - (rect.Width() >> 1) + AXES_SPACE);
}

void CPoleZeroWnd::OnDestroy() 
{
	CFrameWnd::OnDestroy();
	
	// TODO: Add your message handler code here
	if (d != NULL) delete d;
}

void CPoleZeroWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default

	if (m_GraphType == 3) return;
	if (!a_rect.PtInRect(point)) return;

	char s[64];
	int x2 = point.x - a_rect.left - 2;
	if (x2 >= R_LENGTH || x2 < 0) return;
	int x = int(x2 * a_maxx / R_LENGTH);
	double y = d[x2];
	sprintf(s, "%s = %d\n%s = %g", a_xlabel, x, a_ylabel, y);
	MessageBox(s, "    ");
	
	CFrameWnd::OnLButtonDown(nFlags, point);
}
